"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectImportNames = void 0;
const typescript_1 = require("typescript");
function collectImportNames(node) {
    if (typescript_1.isImportDeclaration(node)) {
        if (!node.importClause) {
            return [];
        }
        const { importClause } = node;
        if (importClause.name) {
            // import def from
            return ['default'];
        }
        if (importClause.namedBindings) {
            const { namedBindings } = importClause;
            if (typescript_1.isNamedImports(namedBindings)) {
                // import {a as b, c} from
                return getAllNames(namedBindings.elements);
            }
            else if (typescript_1.isNamespaceImport(namedBindings)) {
                // import * as all from
                return ['*'];
            }
        }
    }
    else if (typescript_1.isExportDeclaration(node)) {
        if (!node.exportClause) {
            // export * from
            return ['*'];
        }
        const { exportClause } = node;
        if (typescript_1.isNamedExports(exportClause)) {
            // export {a as b, c} from
            return getAllNames(exportClause.elements);
        }
    }
    return ['__failed_parse__'];
}
exports.collectImportNames = collectImportNames;
function getString(v) {
    return v.getText();
}
function getAllNames(list) {
    return list.map((v) => getString(v.propertyName || v.name));
}
//# sourceMappingURL=importIds.js.map